"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Tldr_instances, _Tldr_commandMap, _Tldr_commands, _Tldr_lang, _Tldr_cacheDir, _Tldr_indexFile, _Tldr_tmpZipFile, _Tldr_langDir_get, _Tldr_getLangDir, _Tldr_readManWithCMD, _Tldr_syncCache, _Tldr_lock, _Tldr_downloadFile;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tldr = void 0;
const fs = require("fs");
const node_fetch_1 = require("node-fetch");
const Zip = require("adm-zip");
const path = require("path");
const remoteURL = "https://tldr.sh/assets/tldr.zip";
function mkdirp(path) {
    if (!fs.existsSync(path)) {
        fs.mkdirSync(path, { recursive: true });
    }
}
class Tldr {
    constructor(cacheDir, lang = "en") {
        _Tldr_instances.add(this);
        _Tldr_commandMap.set(this, new Map());
        _Tldr_commands.set(this, []);
        _Tldr_lang.set(this, "en");
        _Tldr_cacheDir.set(this, void 0);
        _Tldr_indexFile.set(this, void 0);
        _Tldr_tmpZipFile.set(this, void 0);
        _Tldr_lock.set(this, false);
        // https://stackoverflow.com/a/51302466
        _Tldr_downloadFile.set(this, function (url, path) {
            return __awaiter(this, void 0, void 0, function* () {
                const res = yield (0, node_fetch_1.default)(url);
                const fileStream = fs.createWriteStream(path);
                return yield new Promise((resolve, reject) => {
                    res.body.pipe(fileStream);
                    res.body.on("error", reject);
                    fileStream.on("finish", resolve);
                });
            });
        });
        __classPrivateFieldSet(this, _Tldr_cacheDir, cacheDir, "f");
        __classPrivateFieldSet(this, _Tldr_indexFile, path.join(__classPrivateFieldGet(this, _Tldr_cacheDir, "f"), `index.json`), "f");
        __classPrivateFieldSet(this, _Tldr_tmpZipFile, path.join(__classPrivateFieldGet(this, _Tldr_cacheDir, "f"), `_tldr.zip`), "f");
        mkdirp(__classPrivateFieldGet(this, _Tldr_cacheDir, "f"));
        if (lang != "en") {
            __classPrivateFieldSet(this, _Tldr_lang, lang, "f");
        }
    }
    updateLang(lang) {
        __classPrivateFieldSet(this, _Tldr_lang, lang, "f");
    }
    commands() {
        __classPrivateFieldGet(this, _Tldr_instances, "m", _Tldr_syncCache).call(this);
        return __classPrivateFieldGet(this, _Tldr_commands, "f");
    }
    command(name) {
        __classPrivateFieldGet(this, _Tldr_instances, "m", _Tldr_syncCache).call(this);
        return __classPrivateFieldGet(this, _Tldr_commandMap, "f").get(name);
    }
    // TODO: impl cache logic
    man(cmd) {
        const command = this.command(cmd);
        if (!command)
            return "";
        let lang = null;
        if (!command.language.includes(__classPrivateFieldGet(this, _Tldr_lang, "f"))) {
            if (command.language.includes('en')) { // en is default
                lang = 'en';
            }
            else {
                lang = command.language[0]; // first language
            }
        }
        return __classPrivateFieldGet(this, _Tldr_instances, "m", _Tldr_readManWithCMD).call(this, command, lang);
    }
    updateCache() {
        return __awaiter(this, arguments, void 0, function* (always = false) {
            if (__classPrivateFieldGet(this, _Tldr_lock, "f")) {
                throw new Error("Cache is already updating");
            }
            if (!fs.existsSync(__classPrivateFieldGet(this, _Tldr_indexFile, "f")) || always) {
                __classPrivateFieldSet(this, _Tldr_lock, true, "f");
                try {
                    yield __classPrivateFieldGet(this, _Tldr_downloadFile, "f").call(this, remoteURL, __classPrivateFieldGet(this, _Tldr_tmpZipFile, "f"));
                    // unzip file
                    new Zip(__classPrivateFieldGet(this, _Tldr_tmpZipFile, "f")).extractAllTo(__classPrivateFieldGet(this, _Tldr_cacheDir, "f"), true);
                }
                catch (error) {
                    console.error(error);
                }
                finally {
                    __classPrivateFieldSet(this, _Tldr_lock, false, "f");
                    fs.unlinkSync(__classPrivateFieldGet(this, _Tldr_tmpZipFile, "f"));
                }
            }
            __classPrivateFieldGet(this, _Tldr_instances, "m", _Tldr_syncCache).call(this);
        });
    }
    get hasLock() {
        return __classPrivateFieldGet(this, _Tldr_lock, "f");
    }
}
exports.Tldr = Tldr;
_Tldr_commandMap = new WeakMap(), _Tldr_commands = new WeakMap(), _Tldr_lang = new WeakMap(), _Tldr_cacheDir = new WeakMap(), _Tldr_indexFile = new WeakMap(), _Tldr_tmpZipFile = new WeakMap(), _Tldr_lock = new WeakMap(), _Tldr_downloadFile = new WeakMap(), _Tldr_instances = new WeakSet(), _Tldr_langDir_get = function _Tldr_langDir_get() {
    return __classPrivateFieldGet(this, _Tldr_instances, "m", _Tldr_getLangDir).call(this, __classPrivateFieldGet(this, _Tldr_lang, "f"));
}, _Tldr_getLangDir = function _Tldr_getLangDir(lang) {
    let prefix = 'pages';
    if (lang != 'en') {
        prefix += `.${lang}`;
    }
    return path.join(__classPrivateFieldGet(this, _Tldr_cacheDir, "f"), prefix);
}, _Tldr_readManWithCMD = function _Tldr_readManWithCMD(cmd, lang) {
    const [platform] = cmd.platform;
    let prefix = __classPrivateFieldGet(this, _Tldr_instances, "a", _Tldr_langDir_get);
    if (lang) {
        prefix = __classPrivateFieldGet(this, _Tldr_instances, "m", _Tldr_getLangDir).call(this, lang);
    }
    const md = path.join(prefix, platform, `${cmd.name}.md`);
    const content = fs.readFileSync(md).toString();
    return content;
}, _Tldr_syncCache = function _Tldr_syncCache() {
    if (__classPrivateFieldGet(this, _Tldr_commands, "f").length == 0) {
        const raw = fs.readFileSync(__classPrivateFieldGet(this, _Tldr_indexFile, "f")).toString('utf-8');
        const { commands } = JSON.parse(raw);
        __classPrivateFieldSet(this, _Tldr_commands, commands.map(c => c.name), "f");
        __classPrivateFieldSet(this, _Tldr_commandMap, new Map(commands.map(c => [c.name, c])), "f");
    }
};
// ;(async ()=> {
//   const tldr = new Tldr('zh')
//   console.log("Updating tldr cache...")
//   await tldr.updateCache(false)
//   console.log("Done")
//   const md = tldr.man('gcc')
//   console.log(md)
// })()
//# sourceMappingURL=tldr.js.map